Attribute VB_Name = "modMain"
' -------------------------------------------------------------------------------
'
'  WhackEd3 - A Windows Doom hacking utility
'  Copyright (C) 2003-2009  Dennis Meuwissen
'
'  This program is free software; you can redistribute it and/or modify
'  it under the terms of the GNU General Public License as published by
'  the Free Software Foundation; either version 2 of the License, or
'  (at your option) any later version.
'
'  This program is distributed in the hope that it will be useful,
'  but WITHOUT ANY WARRANTY; without even the implied warranty of
'  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'  GNU General Public License for more details.
'
'  You should have received a copy of the GNU General Public License
'  along with this program; if not, write to the Free Software
'  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'
' -------------------------------------------------------------------------------
'
'   modMain.bas
'   Purpose: Main startup routines, controls the program loading\ending.
'
Option Explicit
Option Compare Binary
Option Base 0


' State editor filter modes
Public Enum enumStateFilterModes
    Filter_Disabled = 0
    Filter_Things
    Filter_Weapons
    Filter_Unused
End Enum


' Lump data (WADList) dictionary
Public lumpDict As Dictionary

' Classes
Public mainWad As clsWad
Public dataWad As clsWad
Public IWADDetect As clsConfiguration
Public appLog As New clsLog
Public FS As New FileSystemObject
Public Dialog As New clsCommonDialog

' Palette in use
Public paletteData As String

' Paths
Public APP_PATH As String
Public DATA_PATH As String
Public TEMP_PATH As String

' Display Beta warning message?
Public Const BETAMESSAGE = True


' Main startup routine, handles the entire initial loading
Public Sub Main()

    Dim filename As String


    modUtil.initCommonControls

    ' Create application path
    APP_PATH = App.Path
    If Right$(APP_PATH, 1) <> "\" Then APP_PATH = APP_PATH & "\"
    
    ' Create data path
    DATA_PATH = getSpecialPath(applicationData) & "\WhackEd3\"
    If Not FS.FolderExists(DATA_PATH) Then FS.CreateFolder DATA_PATH
    
    ' Create temporary path
    TEMP_PATH = getSpecialPath(Temp) & "\"
    

    ' Start logging
    appLog.StartLog DATA_PATH & "whacked3.log", False
    
    ' Load splash form
    Load frmSplash
    frmSplash.Show
    frmSplash.Refresh

    ' See if we need to display the Beta version message
    If BETAMESSAGE = True Then
        MsgBox "This is a beta release of WhackEd3 -- use it at your own risk!", vbExclamation Or vbOKOnly
        frmSplash.Refresh
    End If
    
    ' Set default configuration values
    appLog.Add "Setting program defaults..."
    modConfig.SetConfigDefaults

    ' Load IWAD detect configuration
    appLog.Add "Loading IWAD detect configuration..."
    Set IWADDetect = New clsConfiguration
    IWADDetect.LoadConfiguration APP_PATH & "iwaddetect.cfg"
    If IWADDetect.Root(False).Item("type") <> CFGTYPE_IWADS Then
        MsgBox "Invalid IWAD detect configuration file.", vbCritical Or vbOKOnly
        modMain.Quit
    End If
    
    ' Load program configuration
    appLog.Add "Loading program configuration..."
    modConfig.LoadProgramConfig
    
    ' Load engine configurations
    appLog.Add "Loading engine configurations..."
    modConfig.GetEngineConfigs
    appLog.Add "  Loaded " & ConfigFiles.Count & " configuration files"
    
    ' Load fonts
    appLog.Add "Loaing custom fonts..."
    installFont APP_PATH & "resources\VeraMono.ttf"
    
    ' Load forms
    appLog.Add "Loading forms..."
    Load frmMain
    Load frmEditThings
    Load frmEditStates
    Load frmEditSounds
    Load frmEditStrings
    Load frmEditWeapons
    Load frmEditAmmo
    Load frmEditCheats
    Load frmEditMisc
    Load frmEditPars
    Load frmPropSound
    Load frmPropString
    Load frmSelectSprite
    Load frmSelectState
    Load frmSelectAction
    Load frmOptions
    Load frmAbout
    Load frmNewFile
    Load frmStartup

    ' Init BASS library
    appLog.Add "Initialising BASS library..."
    If BASS_Init(-1, 44100, 0, 0, 0) <> 1 Then appLog.Add "Could not initialise BASS library."

    ' Apply system font
    modUtil.applyDefaultSystemFont
    
    ' Setup the caption of the main form
    frmMain.updateCaption

    ' Apply program settings
    modMain.applyProgramSettings

    ' Unload splash form and signal ready
    Unload frmSplash
    appLog.Add "Ready."

    ' Display the main window
    frmMain.Show
    
    ' Load a file if specified on the commandline
    If modParse.CmdLine("-load") <> vbNullString Then
        filename = StripQuotes(modParse.CmdLine("-load"))
        appLog.Add "Loading patch file " & filename & "..."

        ' Check if the patch file exists
        If Not FS.FileExists(filename) Then
            MsgBox "Patch file specified on commandline does not exist", vbCritical Or vbOKOnly
            modMain.Quit
        End If

        ' Load the patch file
        frmMain.openFile filename
        
    ' Display startup window
    Else
        frmStartup.Show 1

    End If
    
End Sub


' Clean up and end
Public Sub Quit()

    ' Close the IWAD\data WADs
    If Not (mainWad Is Nothing) Then
        Set mainWad = Nothing
        appLog.Add "Closed IWAD"
    End If
    If Not (dataWad Is Nothing) Then
        Set dataWad = Nothing
        appLog.Add "Closed data WAD"
    End If

    ' Save settings
    appLog.Add "Writing configuration file..."
    modConfig.SaveProgramConfig

    ' Free BASS
    appLog.Add "Freeing BASS library..."
    BASS_Free

    ' Unload forms
    appLog.Add "Unloading forms..."
    Unload frmEditThings
    Unload frmEditStates
    Unload frmEditSounds
    Unload frmEditStrings
    Unload frmEditWeapons
    Unload frmEditAmmo
    Unload frmEditCheats
    Unload frmEditMisc
    Unload frmEditPars
    Unload frmOptions
    Unload frmPropSound
    Unload frmPropString
    Unload frmSelectSprite
    Unload frmSelectState
    Unload frmSelectAction
    Unload frmAbout
    Unload frmNewFile
    Unload frmStartup

    ' End log
    appLog.Add "Buh-bye..."
    Set appLog = Nothing

    ' Die.
    End

End Sub


' Apply the configuration settings
Public Sub applyProgramSettings()

    appLog.Add "Applying program settings..."

    ' Update form table data if in editing mode
    If frmMain.enabledMode = True Then modMain.setupFormData

    ' Update saving functions availability
    updateSavingFunctions

End Sub


' Fill all the editing forms with their relevant data.
Public Sub setupFormData()

    frmEditThings.fillStaticData
    frmEditStates.fillStaticData
    frmEditStates.updateStateList
    frmEditSounds.fillStaticData
    frmEditStrings.updateStringList
    frmEditWeapons.fillStaticData
    frmEditAmmo.fillStaticData
    frmEditCheats.fillStaticData
    frmEditMisc.fillStaticData
    frmEditPars.fillStaticData
    frmSelectAction.fillStaticData
    frmSelectSprite.fillStaticData
    frmSelectState.updateStateList

End Sub
